from mock import MagicMock, patch

from pyolite import Pyolite


def test_if_pyolite_object_has_all_attributes():
    mocked_repository = MagicMock()
    mocked_user = MagicMock()

    with patch.multiple('pyolite.pyolite',
                        RepositoryManager=mocked_repository,
                        UserManager=mocked_user):
        pyolite = Pyolite('my_repo')

        assert pyolite.admin_repository == 'my_repo'
        mocked_repository.assert_called_once_with('my_repo')
        mocked_user.assert_called_once_with('my_repo')



from mock import patch, MagicMock, call

from pyolite.models.lists import ListKeys


def test_if_we_commit_after_a_key_append():
    key_path = "tests/fixtures/simple_key.pub"

    mock_file = MagicMock()
    mock_file.__str__ = lambda x: key_path
    mock_path = MagicMock(return_value=mock_file)

    mock_hashlib = MagicMock()
    mock_hashlib.md5.hexdigest.return_value = "HASH"

    mock_user = MagicMock()
    mock_user.path = "path"
    mock_user.name = "test"

    with patch.multiple('pyolite.models.lists.keys', Path=mock_path,
                        hashlib=mock_hashlib):
        keys = ListKeys(mock_user)

        keys.append(key_path)

    mock_path.has_calls([
        call("path", key_path),
        call("path", "keydir", "HASH"),
        call(mock_file, "test"),
    ])

    assert mock_file.isfile.call_count == 1
    assert mock_file.mkdir.call_count == 1

    mock_file.write_file.assert_called_once_with(b'nothing to see here\n', mode='wb')


def test_list_addition():
    mock_user = MagicMock()
    mock_append = MagicMock()

    keys = ListKeys(mock_user)
    keys.append = mock_append

    keys += ['first_key', 'second_key']

    mock_append.has_calls([
        call('first_key'),
        call('second_key'),
    ])


def test_list_remove():
    key = "begin_rsa 1"

    mock_file = MagicMock()
    mock_file.__str__ = lambda x: key
    mock_file.exists.return_value = True
    mock_path = MagicMock(return_value=mock_file)

    mock_hashlib = MagicMock()
    mock_hashlib.md5.hexdigest.return_value = "HASH"

    mock_user = MagicMock()
    mock_user.path = "path"
    mock_user.name = "test"

    with patch.multiple('pyolite.models.lists.keys', Path=mock_path,
                        hashlib=mock_hashlib):
        keys = ListKeys(mock_user)

        keys.remove(key)

        mock_path.has_calls([
            call("path", 'keydir', 'HASH'),
            call(mock_file, "test.pub"),
        ])

        commit_message = "Removed key for user test"
        mock_user.git.commit.has_calls([call(["my_awesome_key"],
                                             commit_message,
                                             action='remove')])



from pyolite import Pyolite

# initial olite object
admin_repository = '/home/absolute/path/to/gitolite/repo/'
olite = Pyolite(admin_repository=admin_repository)

# create user object
vlad = olite.users.create(name='bob',
                          key_path='~/.ssh/second_rsa.pub')

# get user by name
vlad = olite.users.get(name='admin')

# get_or_create django style
vlad = olite.users.get_or_create('alice')

# add new key to user
vlad.keys.append('/path/to/key')
vlad.keys.append('just put the key here')

vlad.keys.remove("my awesome key")

# check if user is admin or not
print vlad.is_admin

# TODO:
# vlad.repos['oxygen'].permissions = 'RW+'



from mock import MagicMock, patch

from pyolite.models.repository import Repository


def test_it_should_be_possible_to_retrieve_by_name_a_repo():
    mocked_users = MagicMock()
    mocked_file = MagicMock()
    mocked_dir = MagicMock()
    mocked_path = MagicMock()

    mocked_dir.isdir.return_value = True
    mocked_file.isdir.return_value = False
    mocked_file.__str__ = lambda x: 'tests/fixtures/get_repo_by_name.conf'

    mocked_path.walk.return_value = [mocked_file, mocked_dir]

    with patch.multiple('pyolite.models.repository',
                        Path=MagicMock(return_value=mocked_path),
                        ListUsers=MagicMock(return_value=mocked_users)):
        repo = Repository.get_by_name('new_one', 'simple_path', 'git')

        assert repo.name == 'new_one'
        assert repo.path == 'simple_path'
        assert repo.git == 'git'
        assert repo.users == mocked_users


def test_if_we_find_only_directories_should_return_none():
    mocked_users = MagicMock()
    mocked_dir = MagicMock()
    mocked_path = MagicMock()

    mocked_dir.isdir.return_value = True

    mocked_path.walk.return_value = [mocked_dir]

    with patch.multiple('pyolite.models.repository',
                        Path=MagicMock(return_value=mocked_path),
                        ListUsers=MagicMock(return_value=mocked_users)):
        repo = Repository.get_by_name('new_one', 'simple_path', 'git')
        assert repo == None



import pytest
from mock import MagicMock, patch

from pyolite.managers.manager import Manager


@pytest.mark.xfail(raises=ValueError)
def test_if_admin_repository_is_not_dir_it_should_raise_ValueError():
    mocked_path = MagicMock()
    mocked_git = MagicMock()

    mocked_path.isdir.return_value = False

    with patch.multiple('pyolite.managers.manager',
                        Path=MagicMock(return_value=mocked_path),
                        Git=MagicMock(return_value=mocked_git)):
        with patch.multiple('pyolite.managers.manager.Manager',
                            __abstractmethods__=set()):
            Manager('/path/to/repo')


def test_get_or_create_method():
    mocked_path = MagicMock()
    mocked_git = MagicMock()

    mocked_get = MagicMock(return_value='user')
    mocked_create = MagicMock()

    Manager.get = mocked_get
    Manager.create = mocked_create

    with patch.multiple('pyolite.managers.manager',
                        Path=MagicMock(return_value=mocked_path),
                        Git=MagicMock(return_value=mocked_git)):
        with patch.multiple('pyolite.managers.manager.Manager',
                            __abstractmethods__=set()):
            manager = Manager('/path/to/admin/repo')

            assert manager.get_or_create('mine', 'key') == 'user'
            mocked_get.assert_called_once_with('mine')

@pytest.mark.xfail(raises=NotImplementedError)
def test_get_abstract_method_method():
    mocked_path = MagicMock()
    mocked_git = MagicMock()

    with patch.multiple('pyolite.managers.manager',
                        Path=MagicMock(return_value=mocked_path),
                        Git=MagicMock(return_value=mocked_git)):
        with patch.multiple('pyolite.managers.manager.Manager',
                            __abstractmethods__=set()):
            manager = Manager('/path/to/admin/repo')
            manager.get('entity')


@pytest.mark.xfail(raises=NotImplementedError)
def test_create_abstract_method_method():
    mocked_path = MagicMock()
    mocked_git = MagicMock()

    with patch.multiple('pyolite.managers.manager',
                        Path=MagicMock(return_value=mocked_path),
                        Git=MagicMock(return_value=mocked_git)):
        with patch.multiple('pyolite.managers.manager.Manager',
                            __abstractmethods__=set()):
            manager = Manager('/path/to/admin/repo')
            manager.create('entity')



import pytest
from mock import MagicMock, patch, call

from pyolite.models.user import User


def set_mocks():
    mocks = {}

    for mock in ['initial_path', 'path', 'file', 'keys', 'git',
                 'first_key',
                 'second_key']:
        mocks[mock] = MagicMock()

    key_path = 'tests/fixtures/second_simple_key.pub'
    mocks['second_key'].__str__ = lambda x: key_path
    mocks['second_key'].isdir.return_value = True

    key_path = 'tests/fixtures/simple_key.pub'
    mocks['first_key'].__str__ = lambda x: key_path
    mocks['first_key'].isdir.return_value = False

    mocks['file'].walk.return_value = [mocks['first_key']]
    mocks['path'].return_value = mocks['file']

    return mocks


def test_if_a_user_can_be_retrieved_by_name():
    mocks = set_mocks()

    with patch.multiple('pyolite.models.user', Path=mocks['path'],
                        ListKeys=mocks['keys']):
        user = User(mocks['initial_path'], mocks['git'], 'vtemian', repos=None,
                    keys=[mocks['first_key']])
        test_user = User.get_by_name('vtemian', mocks['initial_path'],
                                     mocks['git'])

        assert test_user.name == user.name
        assert test_user.repos == user.repos
        assert test_user.keys == user.keys
        assert test_user.path == user.path
        assert test_user.git == user.git

        mocks['path'].has_calls([
            call('path', 'keydir'),
            call('path', 'conf/')
        ])

        assert str(test_user) == '< vtemian >'
        assert repr(test_user) == '< vtemian >'


def test_if_user_is_admin():
    mocks = set_mocks()

    with patch.multiple('pyolite.models.user', Path=mocks['path'],
                        ListKeys=mocks['keys']):
        user = User(mocks['initial_path'], mocks['git'], 'vtemian', repos=None,
                    keys=[mocks['first_key']])
        assert not user.is_admin

        user.repos = ['/path/to/gitolite/admin/gitolite.conf']
        assert user.is_admin


@pytest.mark.xfail(raises=ValueError)
def test_get_user_by_nothing_it_should_raise_value_error():
    mocks = set_mocks()

    with patch.multiple('pyolite.models.user', Path=mocks['path'],
                        ListKeys=mocks['keys']):
        User.get(MagicMock(), mocks['git'], mocks['path'])



from unipath import Path

from pyolite.models.lists import ListKeys


class User(object):
    def __init__(self, path, git, name, **kwargs):
        self.name = name
        self.repos = kwargs.get('repos') or []

        self.path = path
        self.git = git

        self.keys = ListKeys(self, kwargs.get('keys') or [])

    @classmethod
    def get_by_name(cls, name, path, git):
        # get user's keys
        key_path = Path(path, 'keydir')
        keys = [key for key in key_path.walk() if
                key.endswith('%s.pub' % name)]

        # get user's repos
        repos = []
        repos_path = Path(path, 'conf/')
        for repo in repos_path.walk():
            if repo.isdir():
                continue

            with open(str(repo)) as f:
                if name in f.read():
                    repos.append(repo)

        if repos or keys:
            return cls(path, git, name, **{'repos': repos, 'keys': keys})

        return None

    @classmethod
    def get(cls, user, path, git):
        if isinstance(user, str):
            user = User.get_by_name(user, path, git)

        if not isinstance(user, User) or not user:
            message = 'Missing user or invalid type'
            raise ValueError(message)

        return user

    @property
    def is_admin(self):
        for repo in self.repos:
            if 'gitolite.conf' in repo:
                return True
        return False

    def __str__(self):
        return "< %s >" % self.name

    def __repr__(self):
        return "< %s >" % self.name



from mock import MagicMock, patch

from pyolite.repo import Repo


def test_it_should_replace_a_given_string_in_repo_conf():
    mocked_re = MagicMock()
    path = 'tests/fixtures/config.conf'

    mocked_re.sub.return_value = 'another_text'

    with patch.multiple('pyolite.repo', re=mocked_re):
        repo = Repo(path)
        repo.replace('pattern', 'string')

        with open('tests/fixtures/config.conf') as f:
            assert f.read() == 'another_text'

        mocked_re.sub.assert_called_once_with('pattern', 'string',
                                              'another_text')


def test_it_should_retrieve_all_users_from_repo():
    path = 'tests/fixtures/repo_users.conf'
    mocked_path = MagicMock()
    mocked_path.__str__ = lambda x: path

    mocked_path.exists.return_value = True

    mocked_re = MagicMock()
    mocked_user1 = MagicMock()
    mocked_user2 = MagicMock()

    mocked_re.compile('=( *)(\w+)').finditer.return_value = [mocked_user1,
                                                             mocked_user2]
    mocked_user1.group.return_value = 'user1'
    mocked_user2.group.return_value = 'user2'

    with patch.multiple('pyolite.repo', re=mocked_re):
        repo = Repo(mocked_path)
        assert repo.users == ['user1', 'user2']


def test_it_should_write_to_repo_config():
    path = 'tests/fixtures/empty_repo.conf'

    Repo(path).write('some_text')

    with open(path, 'r+') as f:
        assert f.read() == 'some_text'

        f.seek(0)
        f.write('')
        f.truncate()


def test_it_should_overwrite_the_repo_config():
    path = 'tests/fixtures/empty_repo.conf'

    Repo(path).write('some_text')

    Repo(path).overwrite('another_text')

    with open(path, 'r+') as f:
        assert f.read() == 'another_text'

        f.seek(0)
        f.write('')
        f.truncate()


def test_replace_filelocking():
    mocked_re = MagicMock()
    mocked_fcntl = MagicMock()

    mocked_open = MagicMock()
    path = 'tests/fixtures/config.conf'

    with patch('pyolite.repo.open', mocked_open, create=True):
        manager = mocked_open.return_value.__enter__.return_value

        # asserts file locking has been put in place before reading
        manager.read = lambda: ([
            mocked_fcntl.flock.assert_called_once_with(
                manager, mocked_fcntl.LOCK_EX
            ),
            mocked_fcntl.reset_mock()
        ])

        with patch.multiple('pyolite.repo', re=mocked_re,
                            fcntl=mocked_fcntl):
            repo = Repo(path)

            mocked_fcntl.reset_mock()
            repo.replace('pattern', 'string')

            # asserts lock has been removed after operating on file
            mocked_fcntl.flock.assert_called_once_with(manager,
                                                       mocked_fcntl.LOCK_UN)


def test_users_filelocking():
    path = 'tests/fixtures/repo_users.conf'
    mocked_path = MagicMock()
    mocked_path.__str__ = lambda x: path

    mocked_path.exists.return_value = True

    mocked_re = MagicMock()
    mocked_fcntl = MagicMock()
    mocked_open = MagicMock()

    with patch('pyolite.repo.open', mocked_open, create=True):
        manager = mocked_open.return_value.__enter__.return_value

        # asserts file locking has been put in place before reading
        manager.read = lambda: ([
            mocked_fcntl.flock.assert_called_once_with(
                manager, mocked_fcntl.LOCK_EX
            ),
            mocked_fcntl.reset_mock()
        ])

        with patch.multiple('pyolite.repo', re=mocked_re,
                            fcntl=mocked_fcntl):
            repo = Repo(mocked_path)

            mocked_fcntl.reset_mock()
            repo.users

            # asserts lock has been removed after reading
            mocked_fcntl.flock.assert_called_once_with(manager,
                                                       mocked_fcntl.LOCK_UN)


def test_write_filelocking():
    path = 'tests/fixtures/empty_repo.conf'
    mocked_path = MagicMock()
    mocked_path.__str__ = lambda x: path

    mocked_fcntl = MagicMock()
    mocked_open = MagicMock()

    with patch('pyolite.repo.open', mocked_open, create=True):
        manager = mocked_open.return_value.__enter__.return_value

        # asserts file locking has been put in place before writing
        manager.write = lambda text: ([
            mocked_fcntl.flock.assert_called_once_with(
                manager, mocked_fcntl.LOCK_EX
            ),
            mocked_fcntl.reset_mock()
        ])

        with patch.multiple('pyolite.repo', fcntl=mocked_fcntl):
            repo = Repo(path)

            mocked_fcntl.reset_mock()
            repo.write('some_text')

            # asserts lock has been removed after writing
            mocked_fcntl.flock.assert_called_once_with(manager,
                                                       mocked_fcntl.LOCK_UN)


def test_overwrite_filelocking():
    path = 'tests/fixtures/empty_repo.conf'
    mocked_path = MagicMock()
    mocked_path.__str__ = lambda x: path

    mocked_fcntl = MagicMock()
    mocked_open = MagicMock()

    with patch('pyolite.repo.open', mocked_open, create=True):
        manager = mocked_open.return_value.__enter__.return_value

        # asserts file locking has been put in place before writing
        manager.write = lambda text: ([
            mocked_fcntl.flock.assert_called_once_with(
                manager, mocked_fcntl.LOCK_EX
            ),
            mocked_fcntl.reset_mock()
        ])

        with patch.multiple('pyolite.repo', fcntl=mocked_fcntl):
            repo = Repo(path)

            mocked_fcntl.reset_mock()
            repo.overwrite('some_text')

            # asserts lock has been removed after writing
            mocked_fcntl.flock.assert_called_once_with(manager,
                                                       mocked_fcntl.LOCK_UN)



from unipath import Path

from pyolite.models.user import User
from pyolite.repo import Repo

ACCEPTED_PERMISSIONS = set('RW+CD')


def with_user(func):
    def decorated(self, string_user, *args, **kwargs):
        try:
            user = User.get(string_user, self.repository_model.path,
                            self.repository_model.git)
        except ValueError:
            user = User(self.repository_model.path,
                        self.repository_model.git,
                        string_user)
        return func(self, user, *args, **kwargs)

    return decorated


class ListUsers(object):
    def __init__(self, repository):
        self.repository_model = repository
        self.repo = Repo(Path(repository.path,
                              "conf/repos/%s.conf" % repository.name))

    @with_user
    def add(self, user, permission):
        if user.name in self.repo.users:
            raise ValueError('User %s already exists. Please check '
                             'example/repository.py in order to see how you can '
                             'delete or change permissions' % user.name)

        if set(map(lambda permission: permission.upper(), permission)) - \
                ACCEPTED_PERMISSIONS != set([]):
            raise ValueError('Invalid permissions. They must be from %s' %
                             ACCEPTED_PERMISSIONS)

        self.repo.write("    %s     =    %s\n" % (permission, user.name))

        commit_message = 'User %s added to repo %s with permissions: %s' % \
                         (user, self.repository_model.name, permission)
        self.repository_model.git.commit(['conf'], commit_message)

        user.repos.append(self.repo)
        return user

    @with_user
    def edit(self, user, permission):
        pattern = r'(\s*)([RW+DC]*)(\s*)=(\s*)%s' % user.name
        string = r"\n    %s    =    %s" % (permission, user.name)

        self.repo.replace(pattern, string)

        self.repository_model.git.commit(['conf'],
                                         "User %s has %s permission for repository %s" %
                                         (user.name, permission,
                                          self.repository_model.name))
        return user

    @with_user
    def remove(self, user):
        pattern = r'(\s*)([RW+DC]*)(\s*)=(\s*)%s' % user.name
        self.repo.replace(pattern, "")

        self.repository_model.git.commit(['conf'],
                                         "Deleted user %s from repository %s" %
                                         (user.name,
                                          self.repository_model.name))

    @with_user
    def get_or_create(self, user):
        return user

    def set(self, users=None):
        users_serialized = "repo {}\n".format(self.repository_model.name)
        if isinstance(users, dict):
            users = users.iteritems()

        if users:
            for user, permission in users:
                if not hasattr(user, 'name'):
                    user = self.get_or_create(user)

                users_serialized += "    %s     =    %s\n" % (permission,
                                                              user.name)

        self.repo.overwrite(users_serialized)

        users = ", ".join((user for user, permission in users))
        commit_message = "Initialized repository %s with users: %s" % (
            self.repository_model.name, users
        )
        self.repository_model.git.commit(['conf'], commit_message)

    def __iter__(self):
        for user in self._user:
            yield user

    def __getitem__(self, item):
        return self._users[item]

    def __setitem__(self, item, value):
        self._users[item] = value

    def __add__(self, items):
        for item in items:
            self.append(item)

    def __str__(self):
        return "['%s']" % ', '.join(self.repo.users)



from .managers.repository import RepositoryManager
from .managers.user import UserManager


class Pyolite(object):
    def __init__(self, admin_repository):
        self.admin_repository = admin_repository

        self.users = UserManager(admin_repository)
        self.repos = RepositoryManager(admin_repository)



from pyolite import Pyolite

# initial olite object
admin_repository = '/home/absolute/path/to/gitolite/repo/'
olite = Pyolite(admin_repository=admin_repository)

# create a repo
repo = olite.repos.get('my_repo')
repo = olite.repos.create('ydo')
repo = olite.repos.get_or_create('second_repo')

print "Repo's users: %s" % repo.users

user = olite.users.create(name='bob', key_path="~/.ssh/third_rsa.pub")

# add a new user
repo.users.add(olite.users.get('admin'), permission='W+')
repo.users.add('bob', permission='R')

# change user's permissions
repo.users.edit(olite.users.get('admin'), permission='WR+')
repo.users.edit('bob', permission='RCW')

# remove user
repo.users.remove('admin')



from abc import ABCMeta, abstractmethod

import six
from unipath import Path

from pyolite.git import Git


@six.add_metaclass(ABCMeta)
class Manager(object):

    def __init__(self, admin_repository):
        self.path = Path(admin_repository)
        self.git = Git(admin_repository)

        if not self.path.isdir():
            raise ValueError('Admin repository path should point to directory')

    def get_or_create(self, lookup_entity, *args, **kwargs):
        return self.get(lookup_entity) or self.create(lookup_entity, *args,
                                                      **kwargs)

    @abstractmethod
    def get(self, *args, **kwargs):
        raise NotImplementedError("Each manager needs a get method")

    @abstractmethod
    def create(self, *args, **kwargs):
        raise NotImplementedError("Each manager needs a create method")



from mock import MagicMock, patch, call
from nose.tools import raises
from pyolite.models.user import User
from spec import Spec

from pyolite.models.lists.users import ListUsers


class TestUserList(Spec):
    @raises(ValueError)
    def test_it_should_raise_ValueError_if_user_exists_when_we_add_him(self):
        mocked_repo = MagicMock()
        mocked_repository = MagicMock()
        mocked_user = MagicMock()

        mocked_repository.name = 'test_repo'
        mocked_repository.path = 'path'

        mocked_user.get.return_value = MagicMock(name='another_user')
        mocked_repo.users = ['another_user']

        with patch.multiple('pyolite.models.lists.users',
                            Repo=mocked_repo, User=mocked_user):
            repo_users = ListUsers(mocked_repository)
            repo_users.add('test', 'hiRW+')

    @raises(ValueError)
    def test_if_we_add_invalid_permissions_it_should_raise_ValueError(self):
        mocked_repo = MagicMock()
        mocked_repository = MagicMock()
        mocked_user = MagicMock()

        mocked_repository.name = 'test_repo'
        mocked_repository.path = 'path'

        mocked_user.get.return_value = MagicMock(name='another_user')
        mocked_repo.users = ['user']

        with patch.multiple('pyolite.models.lists.users',
                            Repo=mocked_repo, User=mocked_user):
            repo_users = ListUsers(mocked_repository)
            repo_users.add('test', 'hiRW+')

    def test_it_should_add_a_new_user_to_repo_if_is_valid(self):
        mocked_repo = MagicMock()
        mocked_repository = MagicMock()
        mocked_user = MagicMock()

        mock_single_user = MagicMock()
        mock_single_user.name = 'another_user'
        mock_single_user.__str__ = lambda x: 'another_user'

        mocked_repository.name = 'test_repo'
        mocked_repository.path = 'path'

        mocked_user.get.return_value = mock_single_user
        mocked_repo.users = ['user']

        with patch.multiple('pyolite.models.lists.users',
                            Repo=MagicMock(return_value=mocked_repo),
                            User=mocked_user):
            repo_users = ListUsers(mocked_repository)
            repo_users.add('test', 'RW+')

            content = '    RW+     =    another_user\n'
            mocked_repo.write.assert_called_once_with(content)

            message = 'User another_user added to repo test_repo ' \
                      'with permissions: RW+'
            mocked_repository.git.commit.has_calls([call(['conf'], message)])

    def test_user_removing(self):
        mocked_repo = MagicMock()
        mocked_repository = MagicMock()
        mocked_user = MagicMock()

        mock_single_user = MagicMock()
        mock_single_user.name = 'another_user'
        mock_single_user.__str__ = lambda x: 'another_user'

        mocked_repository.name = 'test_repo'
        mocked_repository.path = 'path'

        mocked_user.get.return_value = mock_single_user
        mocked_repo.users = ['user']

        with patch.multiple('pyolite.models.lists.users',
                            Repo=MagicMock(return_value=mocked_repo),
                            User=mocked_user):
            repo_users = ListUsers(mocked_repository)
            repo_users.remove('test')

            pattern = r'(\s*)([RW+DC]*)(\s*)=(\s*)%s' % 'another_user'
            mocked_repo.replace.assert_called_once_with(pattern, "")

            message = "Deleted user another_user from repository test_repo"
            mocked_repository.git.commit.has_calls([call(['conf'], message)])

    def test_user_edit_permissions(self):
        mocked_repo = MagicMock()
        mocked_repository = MagicMock()
        mocked_user = MagicMock()

        mock_single_user = MagicMock()
        mock_single_user.name = 'another_user'
        mock_single_user.__str__ = lambda x: 'another_user'

        mocked_repository.name = 'test_repo'
        mocked_repository.path = 'path'

        mocked_user.get.return_value = mock_single_user
        mocked_repo.users = ['user']

        with patch.multiple('pyolite.models.lists.users',
                            Repo=MagicMock(return_value=mocked_repo),
                            User=mocked_user):
            repo_users = ListUsers(mocked_repository)
            repo_users.edit('test', 'R')

            pattern = r'(\s*)([RW+DC]*)(\s*)=(\s*)%s' % 'another_user'
            string = r"\n    %s    =    %s" % ('R', 'another_user')
            mocked_repo.replace.assert_called_once_with(pattern, string)

            message = "User another_user has R permission for repository test_repo"
            mocked_repository.git.commit.has_calls([call(['conf'], message)])

    def user_init(self, *args, **kwargs):
        self.name = args[2]

    @patch('pyolite.models.user.User.__init__', new=user_init)
    def test_user_get_or_create(self):
        mocked_repo = MagicMock()
        mocked_repository = MagicMock()

        mocked_repository.name = 'test_repo'
        mocked_repository.path = 'path'

        with patch.multiple('pyolite.models.lists.users',
                            Repo=MagicMock(return_value=mocked_repo)):
            found_user = object()
            mocked_user_get = MagicMock(return_value=found_user)

            with patch.multiple('pyolite.models.user.User', get=mocked_user_get):
                repo_users = ListUsers(mocked_repository)

                # user found
                user = repo_users.get_or_create('test_user')
                assert user is found_user

                # user created
                mocked_user_get.side_effect = ValueError
                user = repo_users.get_or_create('test_user')
                assert user.name is 'test_user'

    def test_users_set(self):
        mocked_repo = MagicMock()
        mocked_repository = MagicMock()
        mocked_user = MagicMock()

        mock_single_user = MagicMock()
        mock_single_user.name = 'user'

        mocked_repository.name = 'test_repo'
        mocked_repository.path = 'path'

        mocked_user.get.return_value = mock_single_user

        with patch.multiple('pyolite.models.lists.users',
                            Repo=MagicMock(return_value=mocked_repo),
                            User=mocked_user):
            repo_users = ListUsers(mocked_repository)
            repo_users.set((('mocked', 'R'), ('mocked', 'RW+')))

            serialized_users = "repo test_repo\n    R     =    user\n" \
                               "    RW+     =    user\n"
            mocked_repo.overwrite.assert_called_once_with(serialized_users)

            message = "Initialized repository test_repo with users: test, user"
            mocked_repository.git.commit.has_calls([call(['conf'], message)])



import re
import fcntl


class Repo(object):
    def __init__(self, path):
        self.path = path

    def replace(self, pattern, string):
        with open(str(self.path), 'r+') as f:
            fcntl.flock(f, fcntl.LOCK_EX)

            content = f.read()
            content = re.sub(pattern, string, content)

            f.seek(0)
            f.write(content)
            f.truncate()

            fcntl.flock(f, fcntl.LOCK_UN)

    @property
    def users(self):
        if not self.path.exists():
            return []

        users = []
        with open(str(self.path)) as f:
            fcntl.flock(f, fcntl.LOCK_EX)
            config = f.read()
            fcntl.flock(f, fcntl.LOCK_UN)
            for match in re.compile(r'=( *)(\w+)').finditer(config):
                users.append(match.group(2))

        return users

    def write(self, string):
        with open(self.path, 'a') as f:
            fcntl.flock(f, fcntl.LOCK_EX)
            f.write(string)
            fcntl.flock(f, fcntl.LOCK_UN)

    def overwrite(self, string):
        with open(self.path, 'w') as f:
            fcntl.flock(f, fcntl.LOCK_EX)
            f.write(string)
            fcntl.flock(f, fcntl.LOCK_UN)



from setuptools import setup, find_packages


requires = ['sh==1.09', 'Unipath==1.1', 'argparse==1.2.1',
            'coverage==3.7.1', 'mock==1.0.1', 'nose==1.3.3',
            'six==1.6.1', 'spec==0.11.1']

setup(name="pyolite",
      version='1.5.9',
      platforms='any',
      packages=find_packages(),
      include_package_data=True,
      install_requires=requires,
      author="Vlad Temian",
      author_email="vlad@presslabs.com",
      url="https://github.com/Presslabs/pyolite",
      description="Python wrapper for gitolite",
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Intended Audience :: Developers',
          'Intended Audience :: System Administrators',
          'License :: OSI Approved :: BSD License',
          'Operating System :: MacOS :: MacOS X',
          'Operating System :: Microsoft :: Windows',
          'Operating System :: POSIX',
          'Topic :: System :: Networking',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.5'
      ])



import pytest
from mock import MagicMock, patch, call

from pyolite.managers.user import UserManager
from tests.mocked_manager import MockManager, mocked_git, mocked_path


@pytest.mark.xfail(raises=ValueError)
def test_create_user_with_no_key():
    with patch.multiple('pyolite.managers.manager',
                        Git=MagicMock(),
                        Path=MagicMock()):
        users = UserManager('~/path/to/admin/gitolite/repo')
        users.create('test_username')

def test_create_user_succesfully():
    mocked_user_obj = MagicMock()
    mocked_user = MagicMock(return_value=mocked_user_obj)

    UserManager.__bases__ = (MockManager,)
    with patch.multiple('pyolite.managers.user', User=mocked_user,
                        Manager=MagicMock()):
        users = UserManager('~/path/to/admin/gitolite/repo')

        assert mocked_user_obj == users.create('test_username', 'key_path')
        mocked_user.assert_called_once_with(mocked_path, mocked_git,
                                            'test_username')
        mocked_user_obj.keys.append.assert_called_once_with('key_path')

def test_get_user():
    mocked_user = MagicMock()
    mocked_user.get_by_name.return_value = 'test_user'

    UserManager.__bases__ = (MockManager,)
    with patch.multiple('pyolite.managers.user', User=mocked_user):
        users = UserManager('~/path/to/admin/gitolite/repo')

        assert users.get('test_user') == 'test_user'
        mocked_user.get_by_name.assert_called_once_with('test_user', mocked_path,mocked_git)

def test_get_all_users():
    mocked_key_dir = MagicMock()
    mocked_file = MagicMock()
    mocked_dir = MagicMock()
    mocked_re = MagicMock()

    mocked_user = MagicMock()
    mocked_user.get_by_name.return_value = 'test_user'

    mocked_path.return_value = mocked_key_dir
    mocked_dir.isdir.return_value = True
    mocked_file.isdir.return_value = False
    mocked_file.__str__ = lambda x: 'ok_file'

    mocked_re.compile().findall.return_value = ['file1.pub']

    mocked_key_dir.walk.return_value = [mocked_file, mocked_dir]

    UserManager.__bases__ = (MockManager,)
    with patch.multiple('pyolite.managers.user', User=mocked_user,
                        Path=mocked_path, re=mocked_re):
        users = UserManager('~/path/to/admin/gitolite/repo')

        assert users.all() == ['test_user']
        mocked_path.has_calls([call(mocked_path, 'keydir')])
        assert mocked_key_dir.walk.call_count == 1
        assert mocked_dir.isdir.call_count == 1
        assert mocked_file.isdir.call_count == 1

        mocked_re.compile.has_calls([call(r'(\w.pub)')])
        mocked_re.compile('\w.pub').findall.assert_called_once_with(r'ok_file')

        mocked_user.get_by_name.assert_called_once_with('file1', mocked_path, mocked_git)



from unipath import Path

from pyolite.models.repository import Repository
from pyolite.managers.manager import Manager


class RepositoryManager(Manager):
    def get(self, entity):
        return Repository.get_by_name(entity, self.path, self.git)

    def create(self, entity):
        repo_file = Path(self.path, 'conf/repos/%s.conf' % entity)
        if repo_file.exists():
            raise ValueError('Repository %s already exists' % entity)
        # If there are missing parent paths in the repo path, create them so we don't get IOErrors
        # In the case of a repo having names with slashes (e.g. "username/reponame")
        elif repo_file.parent != Path(""):
            repo_file.parent.mkdir(parents=True)

        repo_file.write_file("repo %s\n" % entity)

        self.git.commit([str(repo_file)], 'Created repo %s' % entity)

        return Repository(entity, self.path, self.git)



import re

from unipath import Path

from pyolite.managers.manager import Manager
from pyolite.models.user import User


class UserManager(Manager):
    def create(self, name, key=None, key_path=None):
        if key is None and key_path is None:
            raise ValueError('You need to specify a key or key_path')

        user = User(self.path, self.git, name)
        user.keys.append(key or key_path)
        return user

    def get(self, name):
        return User.get_by_name(name, self.path, self.git)

    def all(self):
        users = []
        key_dir = Path(self.path, 'keydir')

        for obj in key_dir.walk():
            if obj.isdir():
                continue

            files = re.compile(r'(\w+.pub)').findall(str(obj))
            if files:
                users += files

        return [User.get_by_name(user[:-4], self.path, self.git)
                for user in set(users)]



from __future__ import absolute_import

import os

from sh import git


class Git(object):
    def __init__(self, repo):
        self.repo = repo

    def commit(self, objects, message):
        # validate commit message
        if not message or not isinstance(message, str):
            raise ValueError(
                "Commit message should not be empty or not string")

        env = os.environ.copy()
        env.update({
            'GIT_WORK_TREE': self.repo,
            'GIT_DIR': '%s/.git' % self.repo,
        })

        git.gc("--force", "--prune", _env=env)
        git.checkout("HEAD", _env=env)

        # pull and push from and to the remote
        git.pull("origin", "master", _env=env)

        for obj in objects:
            git.add("-A", obj, _env=env)

        try:
            git.commit("-m", message, _env=env)
        except Exception:
            pass

        git.push(_env=env)



import hashlib

from unipath import Path


class ListKeys(list):
    def __init__(self, user, *args, **kwargs):
        super(ListKeys, self).__init__(*args, **kwargs)
        self.user = user

    def append(self, key):
        key_path = Path(str(key))

        if key_path.isfile():
            with open(str(key_path)) as f:
                key = f.read()

        if not isinstance(key, bytes):
            key = key.encode('utf-8')

        if key in self:
            return

        directory = Path(self.user.path, 'keydir', self.user.name,
                         hashlib.md5(key.strip().split()[1]).hexdigest())
        directory.mkdir(parents=True)

        key_file = Path(directory, "%s.pub" % self.user.name)
        if key_file.exists() and key_file.read_file() == key:
            return

        key_file.write_file(key, mode='wb')

        self.user.git.commit(['keydir'],
                             'Added new key for user %s' % self.user.name)

        super(ListKeys, self).append(key)

    def remove(self, key):
        if not isinstance(key, bytes):
            key = key.encode('utf-8')

        directory = Path(self.user.path, 'keydir', self.user.name,
                         hashlib.md5(key.strip().split()[1]).hexdigest())
        key_file = Path(directory, "%s.pub" % self.user.name)

        if not key_file.exists():
            raise ValueError("Invalid key")

        key_file.remove()
        key_file.parent.rmdir()

        self.user.git.commit(['keydir'],
                             'Removed key for user %s' % self.user.name)

    def __add__(self, keys):
        for key in keys:
            self.append(key)



import pytest
from mock import MagicMock, patch, call

from pyolite.git import Git


@pytest.mark.xfail(raises=ValueError)
def test_commit_with_no_message():
    mock_repo = MagicMock()

    git = Git(mock_repo)
    objects = ['simple_object', 'more_complex_one']

    git.commit(objects, '')


def test_commit_succesfully_with_multiple_objects():
    mock_repo = MagicMock()

    mock_os = MagicMock()
    mock_env = MagicMock()
    mock_os.environ.copy.return_value = mock_env

    mock_git = MagicMock()

    with patch.multiple('pyolite.git', os=mock_os, git=mock_git):
        git = Git(mock_repo)

        objects = ['simple_object', 'more_complex_one']
        commit_message = 'simple commit message'

        git.commit(objects, commit_message)

    mock_os.environ.copy.assert_called_once_with()
    mock_env.update.assert_called_once_with({'GIT_WORK_TREE': mock_repo,
                                             'GIT_DIR': '%s/.git' % mock_repo})
    mock_git.gc.assert_called_once_with('--force', '--prune', _env=mock_env)
    mock_git.checkout.assert_called_once_with('HEAD', _env=mock_env)
    mock_git.pull("origin", "master", _env=mock_env)

    mock_git.add.assert_has_calls([call('-A', objects[0], _env=mock_env),
                                   call('-A', objects[1], _env=mock_env)])

    mock_git.commit.assert_called_once_with('-m', commit_message, _env=mock_env)
    mock_git.push.assert_called_once_with(_env=mock_env)



import pytest
from mock import MagicMock, patch, call

from pyolite.managers.repository import RepositoryManager

from tests.mocked_manager import MockManager, mocked_git, mocked_path


def test_get_repository():
    mocked_repository = MagicMock()
    mocked_repository.get_by_name.return_value = 'my_repo'

    RepositoryManager.__bases__ = (MockManager,)
    with patch.multiple('pyolite.managers.repository',
                        Repository=mocked_repository):
        repos = RepositoryManager('/path/to/admin/repo/')

        assert repos.get('my_repo') == 'my_repo'
        mocked_repository.get_by_name.assert_called_once_with('my_repo',
                                                              mocked_path,
                                                              mocked_git)

@pytest.mark.xfail(raises=ValueError)
def test_create_new_repository_that_already_exists():
    mocked_repository = MagicMock()

    mocked_file = MagicMock()

    mocked_path.return_value = mocked_file
    mocked_file.exists.return_value = True

    RepositoryManager.__bases__ = (MockManager,)
    with patch.multiple('pyolite.managers.repository',
                        Path=mocked_path,
                        Repository=mocked_repository):
        repos = RepositoryManager('/path/to/admin/repo/')
        repos.create('already_exists')

def test_it_should_commit_if_a_new_repository_was_succesfully_created():
    mocked_repository = MagicMock()

    mocked_file = MagicMock()
    mocked_file.__str__ = lambda x: 'dont_exists'

    mocked_file.exists.return_value = False
    mocked_path.return_value = mocked_file

    mocked_repository.return_value = 'new repo'

    RepositoryManager.__bases__ = (MockManager,)

    with patch.multiple('pyolite.managers.repository',
                        Path=mocked_path,
                        Repository=mocked_repository):
        repos = RepositoryManager('/path/to/admin/repo/')
        repo = repos.create('dont_exists')

        mocked_path.has_calls(call(mocked_path,
                                   'conf/repos/dont_exists.conf'))
        assert mocked_file.exists.call_count == 1
        mocked_file.write_file.assert_called_once_with(
            'repo dont_exists\n')
        mocked_git.commit.has_calls(call(['dont_exists'],
                                         'Created repo dont_exists'))
        mocked_repository.assert_called_once_with('dont_exists',
                                                  mocked_path,
                                                  mocked_git)
        assert repo == 'new repo'



from unipath import Path

from pyolite.models.lists import ListUsers


class Repository(object):
    def __init__(self, name, path, git):
        self.name = name
        self.path = path
        self.git = git

        self.users = ListUsers(self)

    @classmethod
    def get_by_name(cls, lookup_repo, path, git):
        for obj in Path(path, 'conf').walk():
            if obj.isdir():
                continue

            with open(str(obj)) as f:
                if "repo %s" % lookup_repo in f.read():
                    return cls(lookup_repo, path, git)
        return None

    def __str__(self):
        return "< %s >" % self.name
